ParamsScreen > Label {
    margin: 1 0 0 1;
    align-horizontal: center;
}

ParamsScreen > Input {
    margin: 1;
}


PartitionList {
    margin: 1 1 0 1;
    max-height: 80%;
    height: auto;
}

PartitionList > ListItem {
    padding: 0 0 0 1;
}


.mounted {
    color: $text-accent;
}

#start-search-button {
    margin: 0 0 0 10;
}


ParamsScreen > Container {
    align: center middle;
    height: auto;
    min-height: 4;
    dock: bottom;
    margin: 1 1 1 1;
}

ParamsScreen > Container > Checkbox {
    margin: 1 0 0 1;
    dock: right;
    background: black 0%;
}